@@ -15,8 +15,8 @@ class User:
     def __setup__(cls):
         super(User, cls).__setup__()
         cls._error_messages.update({
-                'set_passwd_ldap_user': \
-                    'You can not set a password to a ldap user!',
+                'set_passwd_ldap_user': (
+                    'You can not set the password of ldap user "%s".'),
                 })
 
     @staticmethod
@@ -61,23 +61,30 @@ class User:
                 if cls.ldap_search_user(login,
                         con, connection, attrs=[]):
                     find = True
+                    break
         except Exception:
             pass
         if find:
-            cls.raise_user_error('set_passwd_ldap_user')
+            cls.raise_user_error('set_passwd_ldap_user', (login.rec_name,))
 
     @classmethod
-    def create(cls, values):
-        if values.get('password') and 'login' in values:
-            cls._check_passwd_ldap_user([values['login']])
-        return super(User, cls).create(values)
+    def create(cls, vlist):
+        tocheck = []
+        for values in vlist:
+            if values.get('password') and 'login' in values:
+                tocheck.append(values['login'])
+        if tocheck:
+            cls._check_passwd_ldap_user(tocheck)
+        return super(User, cls).create(vlist)
 
     @classmethod
-    def write(cls, users, values):
-        if values.get('password'):
-            logins = [x.login for x in users]
-            cls._check_passwd_ldap_user(logins)
-        super(User, cls).write(users, values)
+    def write(cls, *args):
+        actions = iter(args)
+        for users, values in zip(actions, actions):
+            if values.get('password'):
+                logins = [x.login for x in users]
+                cls._check_passwd_ldap_user(logins)
+        super(User, cls).write(*args)
 
     @classmethod
     def set_preferences(cls, values, old_password=False):
@@ -108,29 +115,33 @@ class User:
 
     @classmethod
     def get_login(cls, login, password):
-        Connection = Pool().get('ldap.connection')
+        pool = Pool()
+        Connection = pool.get('ldap.connection')
+        LoginAttempt = pool.get('res.user.login.attempt')
         with Transaction().set_user(0):
             connection, = Connection.search([], limit=1)
         try:
-            con = ldap.initialize(connection.uri)
+            con = ldap.initialize(connection.uri or '')
             if connection.active_directory:
                 con.set_option(ldap.OPT_REFERRALS, 0)
             if connection.secure == 'tls':
                 con.start_tls_s()
             if connection.bind_dn:
-                con.simple_bind_s(connection.bind_dn, connection.bind_pass)
+                con.simple_bind_s(connection.bind_dn or '',
+                    connection.bind_pass or '')
             [(dn, attrs)] = cls.ldap_search_user(login, con, connection,
                 attrs=[str(connection.auth_uid)])
             if password and con.simple_bind_s(dn, password):
-                user_id, _, _ = cls._get_login(login)
+                user_id, _ = cls._get_login(login)
                 if user_id:
+                    LoginAttempt.remove(login)
                     return user_id
                 elif connection.auth_create_user:
-                    user = cls.create({
-                            'name': attrs.get(str(connection.auth_uid),
-                                [login])[0],
-                            'login': login,
-                            })
+                    user, = cls.create([{
+                                'name': attrs.get(str(connection.auth_uid),
+                                    [login])[0],
+                                'login': login,
+                                }])
                     return user.id
         except Exception:
             pass
